# -*- coding: utf-8 -*-
# This file is auto-generated, don't edit it. Thanks.
from typing import Dict
from Tea.core import TeaCore

from alibabacloud_tea_openapi.client import Client as OpenApiClient
from alibabacloud_tea_openapi import models as open_api_models
from alibabacloud_tea_util.client import Client as UtilClient
from alibabacloud_endpoint_util.client import Client as EndpointUtilClient
from alibabacloud_dg20190327 import models as dg_20190327_models
from alibabacloud_tea_util import models as util_models
from alibabacloud_openapi_util.client import Client as OpenApiUtilClient


class Client(OpenApiClient):
    """
    *\
    """
    def __init__(
        self, 
        config: open_api_models.Config,
    ):
        super().__init__(config)
        self._endpoint_rule = 'regional'
        self._endpoint_map = {
            'ap-northeast-1': 'dg.aliyuncs.com',
            'ap-northeast-2-pop': 'dg.aliyuncs.com',
            'ap-south-1': 'dg.aliyuncs.com',
            'ap-southeast-1': 'dg.aliyuncs.com',
            'ap-southeast-2': 'dg.aliyuncs.com',
            'ap-southeast-3': 'dg.aliyuncs.com',
            'ap-southeast-5': 'dg.aliyuncs.com',
            'cn-beijing': 'dg.aliyuncs.com',
            'cn-beijing-finance-1': 'dg.aliyuncs.com',
            'cn-beijing-finance-pop': 'dg.aliyuncs.com',
            'cn-beijing-gov-1': 'dg.aliyuncs.com',
            'cn-beijing-nu16-b01': 'dg.aliyuncs.com',
            'cn-chengdu': 'dg.aliyuncs.com',
            'cn-edge-1': 'dg.aliyuncs.com',
            'cn-fujian': 'dg.aliyuncs.com',
            'cn-haidian-cm12-c01': 'dg.aliyuncs.com',
            'cn-hangzhou-bj-b01': 'dg.aliyuncs.com',
            'cn-hangzhou-finance': 'dg.aliyuncs.com',
            'cn-hangzhou-internal-prod-1': 'dg.aliyuncs.com',
            'cn-hangzhou-internal-test-1': 'dg.aliyuncs.com',
            'cn-hangzhou-internal-test-2': 'dg.aliyuncs.com',
            'cn-hangzhou-internal-test-3': 'dg.aliyuncs.com',
            'cn-hangzhou-test-306': 'dg.aliyuncs.com',
            'cn-hongkong': 'dg.aliyuncs.com',
            'cn-hongkong-finance-pop': 'dg.aliyuncs.com',
            'cn-huhehaote': 'dg.aliyuncs.com',
            'cn-north-2-gov-1': 'dg.aliyuncs.com',
            'cn-qingdao': 'dg.aliyuncs.com',
            'cn-qingdao-nebula': 'dg.aliyuncs.com',
            'cn-shanghai': 'dg.aliyuncs.com',
            'cn-shanghai-et15-b01': 'dg.aliyuncs.com',
            'cn-shanghai-et2-b01': 'dg.aliyuncs.com',
            'cn-shanghai-finance-1': 'dg.aliyuncs.com',
            'cn-shanghai-inner': 'dg.aliyuncs.com',
            'cn-shanghai-internal-test-1': 'dg.aliyuncs.com',
            'cn-shenzhen': 'dg.aliyuncs.com',
            'cn-shenzhen-finance-1': 'dg.aliyuncs.com',
            'cn-shenzhen-inner': 'dg.aliyuncs.com',
            'cn-shenzhen-st4-d01': 'dg.aliyuncs.com',
            'cn-shenzhen-su18-b01': 'dg.aliyuncs.com',
            'cn-wuhan': 'dg.aliyuncs.com',
            'cn-yushanfang': 'dg.aliyuncs.com',
            'cn-zhangbei-na61-b01': 'dg.aliyuncs.com',
            'cn-zhangjiakou': 'dg.aliyuncs.com',
            'cn-zhangjiakou-na62-a01': 'dg.aliyuncs.com',
            'cn-zhengzhou-nebula-1': 'dg.aliyuncs.com',
            'eu-central-1': 'dg.aliyuncs.com',
            'eu-west-1': 'dg.aliyuncs.com',
            'eu-west-1-oxs': 'dg.aliyuncs.com',
            'me-east-1': 'dg.aliyuncs.com',
            'rus-west-1-pop': 'dg.aliyuncs.com',
            'us-east-1': 'dg.aliyuncs.com',
            'us-west-1': 'dg.aliyuncs.com'
        }
        self.check_config(config)
        self._endpoint = self.get_endpoint('dg', self._region_id, self._endpoint_rule, self._network, self._suffix, self._endpoint_map, self._endpoint)

    def get_endpoint(
        self,
        product_id: str,
        region_id: str,
        endpoint_rule: str,
        network: str,
        suffix: str,
        endpoint_map: Dict[str, str],
        endpoint: str,
    ) -> str:
        if not UtilClient.empty(endpoint):
            return endpoint
        if not UtilClient.is_unset(endpoint_map) and not UtilClient.empty(endpoint_map.get(region_id)):
            return endpoint_map.get(region_id)
        return EndpointUtilClient.get_endpoint_rules(product_id, region_id, endpoint_rule, network, suffix)

    def add_database_with_options(
        self,
        request: dg_20190327_models.AddDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.AddDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='AddDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.AddDatabaseResponse(),
            self.call_api(params, req, runtime)
        )

    async def add_database_with_options_async(
        self,
        request: dg_20190327_models.AddDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.AddDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='AddDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.AddDatabaseResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def add_database(
        self,
        request: dg_20190327_models.AddDatabaseRequest,
    ) -> dg_20190327_models.AddDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return self.add_database_with_options(request, runtime)

    async def add_database_async(
        self,
        request: dg_20190327_models.AddDatabaseRequest,
    ) -> dg_20190327_models.AddDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return await self.add_database_with_options_async(request, runtime)

    def add_database_list_with_options(
        self,
        request: dg_20190327_models.AddDatabaseListRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.AddDatabaseListResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.database_string):
            body['DatabaseString'] = request.database_string
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='AddDatabaseList',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.AddDatabaseListResponse(),
            self.call_api(params, req, runtime)
        )

    async def add_database_list_with_options_async(
        self,
        request: dg_20190327_models.AddDatabaseListRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.AddDatabaseListResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.database_string):
            body['DatabaseString'] = request.database_string
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='AddDatabaseList',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.AddDatabaseListResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def add_database_list(
        self,
        request: dg_20190327_models.AddDatabaseListRequest,
    ) -> dg_20190327_models.AddDatabaseListResponse:
        runtime = util_models.RuntimeOptions()
        return self.add_database_list_with_options(request, runtime)

    async def add_database_list_async(
        self,
        request: dg_20190327_models.AddDatabaseListRequest,
    ) -> dg_20190327_models.AddDatabaseListResponse:
        runtime = util_models.RuntimeOptions()
        return await self.add_database_list_with_options_async(request, runtime)

    def connect_database_with_options(
        self,
        request: dg_20190327_models.ConnectDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ConnectDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ConnectDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ConnectDatabaseResponse(),
            self.call_api(params, req, runtime)
        )

    async def connect_database_with_options_async(
        self,
        request: dg_20190327_models.ConnectDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ConnectDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ConnectDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ConnectDatabaseResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def connect_database(
        self,
        request: dg_20190327_models.ConnectDatabaseRequest,
    ) -> dg_20190327_models.ConnectDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return self.connect_database_with_options(request, runtime)

    async def connect_database_async(
        self,
        request: dg_20190327_models.ConnectDatabaseRequest,
    ) -> dg_20190327_models.ConnectDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return await self.connect_database_with_options_async(request, runtime)

    def create_database_access_point_with_options(
        self,
        request: dg_20190327_models.CreateDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.v_switch_id):
            body['VSwitchId'] = request.v_switch_id
        if not UtilClient.is_unset(request.vpc_azone):
            body['VpcAZone'] = request.vpc_azone
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        if not UtilClient.is_unset(request.vpc_region_id):
            body['VpcRegionId'] = request.vpc_region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateDatabaseAccessPointResponse(),
            self.call_api(params, req, runtime)
        )

    async def create_database_access_point_with_options_async(
        self,
        request: dg_20190327_models.CreateDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.v_switch_id):
            body['VSwitchId'] = request.v_switch_id
        if not UtilClient.is_unset(request.vpc_azone):
            body['VpcAZone'] = request.vpc_azone
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        if not UtilClient.is_unset(request.vpc_region_id):
            body['VpcRegionId'] = request.vpc_region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateDatabaseAccessPointResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def create_database_access_point(
        self,
        request: dg_20190327_models.CreateDatabaseAccessPointRequest,
    ) -> dg_20190327_models.CreateDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return self.create_database_access_point_with_options(request, runtime)

    async def create_database_access_point_async(
        self,
        request: dg_20190327_models.CreateDatabaseAccessPointRequest,
    ) -> dg_20190327_models.CreateDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return await self.create_database_access_point_with_options_async(request, runtime)

    def create_gateway_with_options(
        self,
        request: dg_20190327_models.CreateGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_desc):
            body['GatewayDesc'] = request.gateway_desc
        if not UtilClient.is_unset(request.gateway_name):
            body['GatewayName'] = request.gateway_name
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateGatewayResponse(),
            self.call_api(params, req, runtime)
        )

    async def create_gateway_with_options_async(
        self,
        request: dg_20190327_models.CreateGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_desc):
            body['GatewayDesc'] = request.gateway_desc
        if not UtilClient.is_unset(request.gateway_name):
            body['GatewayName'] = request.gateway_name
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateGatewayResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def create_gateway(
        self,
        request: dg_20190327_models.CreateGatewayRequest,
    ) -> dg_20190327_models.CreateGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return self.create_gateway_with_options(request, runtime)

    async def create_gateway_async(
        self,
        request: dg_20190327_models.CreateGatewayRequest,
    ) -> dg_20190327_models.CreateGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return await self.create_gateway_with_options_async(request, runtime)

    def create_gateway_verify_code_with_options(
        self,
        request: dg_20190327_models.CreateGatewayVerifyCodeRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateGatewayVerifyCodeResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateGatewayVerifyCode',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateGatewayVerifyCodeResponse(),
            self.call_api(params, req, runtime)
        )

    async def create_gateway_verify_code_with_options_async(
        self,
        request: dg_20190327_models.CreateGatewayVerifyCodeRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.CreateGatewayVerifyCodeResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='CreateGatewayVerifyCode',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.CreateGatewayVerifyCodeResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def create_gateway_verify_code(
        self,
        request: dg_20190327_models.CreateGatewayVerifyCodeRequest,
    ) -> dg_20190327_models.CreateGatewayVerifyCodeResponse:
        runtime = util_models.RuntimeOptions()
        return self.create_gateway_verify_code_with_options(request, runtime)

    async def create_gateway_verify_code_async(
        self,
        request: dg_20190327_models.CreateGatewayVerifyCodeRequest,
    ) -> dg_20190327_models.CreateGatewayVerifyCodeResponse:
        runtime = util_models.RuntimeOptions()
        return await self.create_gateway_verify_code_with_options_async(request, runtime)

    def delete_database_with_options(
        self,
        request: dg_20190327_models.DeleteDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteDatabaseResponse(),
            self.call_api(params, req, runtime)
        )

    async def delete_database_with_options_async(
        self,
        request: dg_20190327_models.DeleteDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteDatabaseResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def delete_database(
        self,
        request: dg_20190327_models.DeleteDatabaseRequest,
    ) -> dg_20190327_models.DeleteDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return self.delete_database_with_options(request, runtime)

    async def delete_database_async(
        self,
        request: dg_20190327_models.DeleteDatabaseRequest,
    ) -> dg_20190327_models.DeleteDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return await self.delete_database_with_options_async(request, runtime)

    def delete_database_access_point_with_options(
        self,
        request: dg_20190327_models.DeleteDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.v_switch_id):
            body['VSwitchId'] = request.v_switch_id
        if not UtilClient.is_unset(request.vpc_azone):
            body['VpcAZone'] = request.vpc_azone
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        if not UtilClient.is_unset(request.vpc_region_id):
            body['VpcRegionId'] = request.vpc_region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteDatabaseAccessPointResponse(),
            self.call_api(params, req, runtime)
        )

    async def delete_database_access_point_with_options_async(
        self,
        request: dg_20190327_models.DeleteDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.v_switch_id):
            body['VSwitchId'] = request.v_switch_id
        if not UtilClient.is_unset(request.vpc_azone):
            body['VpcAZone'] = request.vpc_azone
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        if not UtilClient.is_unset(request.vpc_region_id):
            body['VpcRegionId'] = request.vpc_region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteDatabaseAccessPointResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def delete_database_access_point(
        self,
        request: dg_20190327_models.DeleteDatabaseAccessPointRequest,
    ) -> dg_20190327_models.DeleteDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return self.delete_database_access_point_with_options(request, runtime)

    async def delete_database_access_point_async(
        self,
        request: dg_20190327_models.DeleteDatabaseAccessPointRequest,
    ) -> dg_20190327_models.DeleteDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return await self.delete_database_access_point_with_options_async(request, runtime)

    def delete_gateway_with_options(
        self,
        request: dg_20190327_models.DeleteGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteGatewayResponse(),
            self.call_api(params, req, runtime)
        )

    async def delete_gateway_with_options_async(
        self,
        request: dg_20190327_models.DeleteGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteGatewayResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def delete_gateway(
        self,
        request: dg_20190327_models.DeleteGatewayRequest,
    ) -> dg_20190327_models.DeleteGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return self.delete_gateway_with_options(request, runtime)

    async def delete_gateway_async(
        self,
        request: dg_20190327_models.DeleteGatewayRequest,
    ) -> dg_20190327_models.DeleteGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return await self.delete_gateway_with_options_async(request, runtime)

    def delete_gateway_instance_with_options(
        self,
        request: dg_20190327_models.DeleteGatewayInstanceRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteGatewayInstanceResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_instance_id):
            body['GatewayInstanceId'] = request.gateway_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteGatewayInstance',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteGatewayInstanceResponse(),
            self.call_api(params, req, runtime)
        )

    async def delete_gateway_instance_with_options_async(
        self,
        request: dg_20190327_models.DeleteGatewayInstanceRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DeleteGatewayInstanceResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_instance_id):
            body['GatewayInstanceId'] = request.gateway_instance_id
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DeleteGatewayInstance',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DeleteGatewayInstanceResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def delete_gateway_instance(
        self,
        request: dg_20190327_models.DeleteGatewayInstanceRequest,
    ) -> dg_20190327_models.DeleteGatewayInstanceResponse:
        runtime = util_models.RuntimeOptions()
        return self.delete_gateway_instance_with_options(request, runtime)

    async def delete_gateway_instance_async(
        self,
        request: dg_20190327_models.DeleteGatewayInstanceRequest,
    ) -> dg_20190327_models.DeleteGatewayInstanceResponse:
        runtime = util_models.RuntimeOptions()
        return await self.delete_gateway_instance_with_options_async(request, runtime)

    def describe_regions_with_options(
        self,
        request: dg_20190327_models.DescribeRegionsRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DescribeRegionsResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DescribeRegions',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DescribeRegionsResponse(),
            self.call_api(params, req, runtime)
        )

    async def describe_regions_with_options_async(
        self,
        request: dg_20190327_models.DescribeRegionsRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DescribeRegionsResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DescribeRegions',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DescribeRegionsResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def describe_regions(
        self,
        request: dg_20190327_models.DescribeRegionsRequest,
    ) -> dg_20190327_models.DescribeRegionsResponse:
        runtime = util_models.RuntimeOptions()
        return self.describe_regions_with_options(request, runtime)

    async def describe_regions_async(
        self,
        request: dg_20190327_models.DescribeRegionsRequest,
    ) -> dg_20190327_models.DescribeRegionsResponse:
        runtime = util_models.RuntimeOptions()
        return await self.describe_regions_with_options_async(request, runtime)

    def download_gateway_program_with_options(
        self,
        request: dg_20190327_models.DownloadGatewayProgramRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DownloadGatewayProgramResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.dg_version):
            body['DgVersion'] = request.dg_version
        if not UtilClient.is_unset(request.user_os):
            body['UserOS'] = request.user_os
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DownloadGatewayProgram',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DownloadGatewayProgramResponse(),
            self.call_api(params, req, runtime)
        )

    async def download_gateway_program_with_options_async(
        self,
        request: dg_20190327_models.DownloadGatewayProgramRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.DownloadGatewayProgramResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.dg_version):
            body['DgVersion'] = request.dg_version
        if not UtilClient.is_unset(request.user_os):
            body['UserOS'] = request.user_os
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='DownloadGatewayProgram',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.DownloadGatewayProgramResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def download_gateway_program(
        self,
        request: dg_20190327_models.DownloadGatewayProgramRequest,
    ) -> dg_20190327_models.DownloadGatewayProgramResponse:
        runtime = util_models.RuntimeOptions()
        return self.download_gateway_program_with_options(request, runtime)

    async def download_gateway_program_async(
        self,
        request: dg_20190327_models.DownloadGatewayProgramRequest,
    ) -> dg_20190327_models.DownloadGatewayProgramResponse:
        runtime = util_models.RuntimeOptions()
        return await self.download_gateway_program_with_options_async(request, runtime)

    def find_user_gateway_by_id_with_options(
        self,
        request: dg_20190327_models.FindUserGatewayByIdRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.FindUserGatewayByIdResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='FindUserGatewayById',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.FindUserGatewayByIdResponse(),
            self.call_api(params, req, runtime)
        )

    async def find_user_gateway_by_id_with_options_async(
        self,
        request: dg_20190327_models.FindUserGatewayByIdRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.FindUserGatewayByIdResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='FindUserGatewayById',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.FindUserGatewayByIdResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def find_user_gateway_by_id(
        self,
        request: dg_20190327_models.FindUserGatewayByIdRequest,
    ) -> dg_20190327_models.FindUserGatewayByIdResponse:
        runtime = util_models.RuntimeOptions()
        return self.find_user_gateway_by_id_with_options(request, runtime)

    async def find_user_gateway_by_id_async(
        self,
        request: dg_20190327_models.FindUserGatewayByIdRequest,
    ) -> dg_20190327_models.FindUserGatewayByIdResponse:
        runtime = util_models.RuntimeOptions()
        return await self.find_user_gateway_by_id_with_options_async(request, runtime)

    def get_user_databases_with_options(
        self,
        request: dg_20190327_models.GetUserDatabasesRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserDatabasesResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserDatabases',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserDatabasesResponse(),
            self.call_api(params, req, runtime)
        )

    async def get_user_databases_with_options_async(
        self,
        request: dg_20190327_models.GetUserDatabasesRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserDatabasesResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserDatabases',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserDatabasesResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def get_user_databases(
        self,
        request: dg_20190327_models.GetUserDatabasesRequest,
    ) -> dg_20190327_models.GetUserDatabasesResponse:
        runtime = util_models.RuntimeOptions()
        return self.get_user_databases_with_options(request, runtime)

    async def get_user_databases_async(
        self,
        request: dg_20190327_models.GetUserDatabasesRequest,
    ) -> dg_20190327_models.GetUserDatabasesResponse:
        runtime = util_models.RuntimeOptions()
        return await self.get_user_databases_with_options_async(request, runtime)

    def get_user_gateway_instances_with_options(
        self,
        request: dg_20190327_models.GetUserGatewayInstancesRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserGatewayInstancesResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserGatewayInstances',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserGatewayInstancesResponse(),
            self.call_api(params, req, runtime)
        )

    async def get_user_gateway_instances_with_options_async(
        self,
        request: dg_20190327_models.GetUserGatewayInstancesRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserGatewayInstancesResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserGatewayInstances',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserGatewayInstancesResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def get_user_gateway_instances(
        self,
        request: dg_20190327_models.GetUserGatewayInstancesRequest,
    ) -> dg_20190327_models.GetUserGatewayInstancesResponse:
        runtime = util_models.RuntimeOptions()
        return self.get_user_gateway_instances_with_options(request, runtime)

    async def get_user_gateway_instances_async(
        self,
        request: dg_20190327_models.GetUserGatewayInstancesRequest,
    ) -> dg_20190327_models.GetUserGatewayInstancesResponse:
        runtime = util_models.RuntimeOptions()
        return await self.get_user_gateway_instances_with_options_async(request, runtime)

    def get_user_gateways_with_options(
        self,
        request: dg_20190327_models.GetUserGatewaysRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserGatewaysResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserGateways',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserGatewaysResponse(),
            self.call_api(params, req, runtime)
        )

    async def get_user_gateways_with_options_async(
        self,
        request: dg_20190327_models.GetUserGatewaysRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.GetUserGatewaysResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='GetUserGateways',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.GetUserGatewaysResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def get_user_gateways(
        self,
        request: dg_20190327_models.GetUserGatewaysRequest,
    ) -> dg_20190327_models.GetUserGatewaysResponse:
        runtime = util_models.RuntimeOptions()
        return self.get_user_gateways_with_options(request, runtime)

    async def get_user_gateways_async(
        self,
        request: dg_20190327_models.GetUserGatewaysRequest,
    ) -> dg_20190327_models.GetUserGatewaysResponse:
        runtime = util_models.RuntimeOptions()
        return await self.get_user_gateways_with_options_async(request, runtime)

    def list_database_access_point_with_options(
        self,
        request: dg_20190327_models.ListDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ListDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ListDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ListDatabaseAccessPointResponse(),
            self.call_api(params, req, runtime)
        )

    async def list_database_access_point_with_options_async(
        self,
        request: dg_20190327_models.ListDatabaseAccessPointRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ListDatabaseAccessPointResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_instance_id):
            body['DbInstanceId'] = request.db_instance_id
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.page_number):
            body['PageNumber'] = request.page_number
        if not UtilClient.is_unset(request.page_size):
            body['PageSize'] = request.page_size
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        if not UtilClient.is_unset(request.search_key):
            body['SearchKey'] = request.search_key
        if not UtilClient.is_unset(request.vpc_id):
            body['VpcId'] = request.vpc_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ListDatabaseAccessPoint',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ListDatabaseAccessPointResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def list_database_access_point(
        self,
        request: dg_20190327_models.ListDatabaseAccessPointRequest,
    ) -> dg_20190327_models.ListDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return self.list_database_access_point_with_options(request, runtime)

    async def list_database_access_point_async(
        self,
        request: dg_20190327_models.ListDatabaseAccessPointRequest,
    ) -> dg_20190327_models.ListDatabaseAccessPointResponse:
        runtime = util_models.RuntimeOptions()
        return await self.list_database_access_point_with_options_async(request, runtime)

    def modify_database_with_options(
        self,
        request: dg_20190327_models.ModifyDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ModifyDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ModifyDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ModifyDatabaseResponse(),
            self.call_api(params, req, runtime)
        )

    async def modify_database_with_options_async(
        self,
        request: dg_20190327_models.ModifyDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ModifyDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.instance_id):
            body['InstanceId'] = request.instance_id
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ModifyDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ModifyDatabaseResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def modify_database(
        self,
        request: dg_20190327_models.ModifyDatabaseRequest,
    ) -> dg_20190327_models.ModifyDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return self.modify_database_with_options(request, runtime)

    async def modify_database_async(
        self,
        request: dg_20190327_models.ModifyDatabaseRequest,
    ) -> dg_20190327_models.ModifyDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return await self.modify_database_with_options_async(request, runtime)

    def modify_gateway_with_options(
        self,
        request: dg_20190327_models.ModifyGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ModifyGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_desc):
            body['GatewayDesc'] = request.gateway_desc
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_name):
            body['GatewayName'] = request.gateway_name
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ModifyGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ModifyGatewayResponse(),
            self.call_api(params, req, runtime)
        )

    async def modify_gateway_with_options_async(
        self,
        request: dg_20190327_models.ModifyGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.ModifyGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_desc):
            body['GatewayDesc'] = request.gateway_desc
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_name):
            body['GatewayName'] = request.gateway_name
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='ModifyGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.ModifyGatewayResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def modify_gateway(
        self,
        request: dg_20190327_models.ModifyGatewayRequest,
    ) -> dg_20190327_models.ModifyGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return self.modify_gateway_with_options(request, runtime)

    async def modify_gateway_async(
        self,
        request: dg_20190327_models.ModifyGatewayRequest,
    ) -> dg_20190327_models.ModifyGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return await self.modify_gateway_with_options_async(request, runtime)

    def retry_database_with_options(
        self,
        request: dg_20190327_models.RetryDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.RetryDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='RetryDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.RetryDatabaseResponse(),
            self.call_api(params, req, runtime)
        )

    async def retry_database_with_options_async(
        self,
        request: dg_20190327_models.RetryDatabaseRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.RetryDatabaseResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.client_token):
            body['ClientToken'] = request.client_token
        if not UtilClient.is_unset(request.db_description):
            body['DbDescription'] = request.db_description
        if not UtilClient.is_unset(request.db_name):
            body['DbName'] = request.db_name
        if not UtilClient.is_unset(request.db_password):
            body['DbPassword'] = request.db_password
        if not UtilClient.is_unset(request.db_type):
            body['DbType'] = request.db_type
        if not UtilClient.is_unset(request.db_user_name):
            body['DbUserName'] = request.db_user_name
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.host):
            body['Host'] = request.host
        if not UtilClient.is_unset(request.port):
            body['Port'] = request.port
        if not UtilClient.is_unset(request.region_id):
            body['RegionId'] = request.region_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='RetryDatabase',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.RetryDatabaseResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def retry_database(
        self,
        request: dg_20190327_models.RetryDatabaseRequest,
    ) -> dg_20190327_models.RetryDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return self.retry_database_with_options(request, runtime)

    async def retry_database_async(
        self,
        request: dg_20190327_models.RetryDatabaseRequest,
    ) -> dg_20190327_models.RetryDatabaseResponse:
        runtime = util_models.RuntimeOptions()
        return await self.retry_database_with_options_async(request, runtime)

    def stop_gateway_with_options(
        self,
        request: dg_20190327_models.StopGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.StopGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_instance_id):
            body['GatewayInstanceId'] = request.gateway_instance_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='StopGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.StopGatewayResponse(),
            self.call_api(params, req, runtime)
        )

    async def stop_gateway_with_options_async(
        self,
        request: dg_20190327_models.StopGatewayRequest,
        runtime: util_models.RuntimeOptions,
    ) -> dg_20190327_models.StopGatewayResponse:
        UtilClient.validate_model(request)
        body = {}
        if not UtilClient.is_unset(request.gateway_id):
            body['GatewayId'] = request.gateway_id
        if not UtilClient.is_unset(request.gateway_instance_id):
            body['GatewayInstanceId'] = request.gateway_instance_id
        req = open_api_models.OpenApiRequest(
            body=OpenApiUtilClient.parse_to_map(body)
        )
        params = open_api_models.Params(
            action='StopGateway',
            version='2019-03-27',
            protocol='HTTPS',
            pathname='/',
            method='POST',
            auth_type='AK',
            style='RPC',
            req_body_type='formData',
            body_type='json'
        )
        return TeaCore.from_map(
            dg_20190327_models.StopGatewayResponse(),
            await self.call_api_async(params, req, runtime)
        )

    def stop_gateway(
        self,
        request: dg_20190327_models.StopGatewayRequest,
    ) -> dg_20190327_models.StopGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return self.stop_gateway_with_options(request, runtime)

    async def stop_gateway_async(
        self,
        request: dg_20190327_models.StopGatewayRequest,
    ) -> dg_20190327_models.StopGatewayResponse:
        runtime = util_models.RuntimeOptions()
        return await self.stop_gateway_with_options_async(request, runtime)



# -*- coding: utf-8 -*-
# This file is auto-generated, don't edit it. Thanks.
from Tea.model import TeaModel
from typing import Dict, List


class AddDatabaseRequest(TeaModel):
    def __init__(
        self,
        client_token: str = None,
        db_description: str = None,
        db_name: str = None,
        db_password: str = None,
        db_type: str = None,
        db_user_name: str = None,
        gateway_id: str = None,
        host: str = None,
        port: int = None,
        region_id: str = None,
    ):
        self.client_token = client_token
        self.db_description = db_description
        self.db_name = db_name
        self.db_password = db_password
        self.db_type = db_type
        self.db_user_name = db_user_name
        self.gateway_id = gateway_id
        self.host = host
        self.port = port
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.client_token is not None:
            result['ClientToken'] = self.client_token
        if self.db_description is not None:
            result['DbDescription'] = self.db_description
        if self.db_name is not None:
            result['DbName'] = self.db_name
        if self.db_password is not None:
            result['DbPassword'] = self.db_password
        if self.db_type is not None:
            result['DbType'] = self.db_type
        if self.db_user_name is not None:
            result['DbUserName'] = self.db_user_name
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.host is not None:
            result['Host'] = self.host
        if self.port is not None:
            result['Port'] = self.port
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('ClientToken') is not None:
            self.client_token = m.get('ClientToken')
        if m.get('DbDescription') is not None:
            self.db_description = m.get('DbDescription')
        if m.get('DbName') is not None:
            self.db_name = m.get('DbName')
        if m.get('DbPassword') is not None:
            self.db_password = m.get('DbPassword')
        if m.get('DbType') is not None:
            self.db_type = m.get('DbType')
        if m.get('DbUserName') is not None:
            self.db_user_name = m.get('DbUserName')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class AddDatabaseResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class AddDatabaseResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: AddDatabaseResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = AddDatabaseResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class AddDatabaseListRequest(TeaModel):
    def __init__(
        self,
        client_token: str = None,
        database_string: str = None,
    ):
        self.client_token = client_token
        self.database_string = database_string

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.client_token is not None:
            result['ClientToken'] = self.client_token
        if self.database_string is not None:
            result['DatabaseString'] = self.database_string
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('ClientToken') is not None:
            self.client_token = m.get('ClientToken')
        if m.get('DatabaseString') is not None:
            self.database_string = m.get('DatabaseString')
        return self


class AddDatabaseListResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class AddDatabaseListResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: AddDatabaseListResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = AddDatabaseListResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class ConnectDatabaseRequest(TeaModel):
    def __init__(
        self,
        db_name: str = None,
        db_password: str = None,
        db_type: str = None,
        db_user_name: str = None,
        gateway_id: str = None,
        host: str = None,
        port: int = None,
    ):
        self.db_name = db_name
        self.db_password = db_password
        self.db_type = db_type
        self.db_user_name = db_user_name
        self.gateway_id = gateway_id
        self.host = host
        self.port = port

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.db_name is not None:
            result['DbName'] = self.db_name
        if self.db_password is not None:
            result['DbPassword'] = self.db_password
        if self.db_type is not None:
            result['DbType'] = self.db_type
        if self.db_user_name is not None:
            result['DbUserName'] = self.db_user_name
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.host is not None:
            result['Host'] = self.host
        if self.port is not None:
            result['Port'] = self.port
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DbName') is not None:
            self.db_name = m.get('DbName')
        if m.get('DbPassword') is not None:
            self.db_password = m.get('DbPassword')
        if m.get('DbType') is not None:
            self.db_type = m.get('DbType')
        if m.get('DbUserName') is not None:
            self.db_user_name = m.get('DbUserName')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        return self


class ConnectDatabaseResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class ConnectDatabaseResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: ConnectDatabaseResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = ConnectDatabaseResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class CreateDatabaseAccessPointRequest(TeaModel):
    def __init__(
        self,
        client_token: str = None,
        db_instance_id: str = None,
        region_id: str = None,
        v_switch_id: str = None,
        vpc_azone: str = None,
        vpc_id: str = None,
        vpc_region_id: str = None,
    ):
        self.client_token = client_token
        self.db_instance_id = db_instance_id
        self.region_id = region_id
        self.v_switch_id = v_switch_id
        self.vpc_azone = vpc_azone
        self.vpc_id = vpc_id
        self.vpc_region_id = vpc_region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.client_token is not None:
            result['ClientToken'] = self.client_token
        if self.db_instance_id is not None:
            result['DbInstanceId'] = self.db_instance_id
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        if self.v_switch_id is not None:
            result['VSwitchId'] = self.v_switch_id
        if self.vpc_azone is not None:
            result['VpcAZone'] = self.vpc_azone
        if self.vpc_id is not None:
            result['VpcId'] = self.vpc_id
        if self.vpc_region_id is not None:
            result['VpcRegionId'] = self.vpc_region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('ClientToken') is not None:
            self.client_token = m.get('ClientToken')
        if m.get('DbInstanceId') is not None:
            self.db_instance_id = m.get('DbInstanceId')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        if m.get('VSwitchId') is not None:
            self.v_switch_id = m.get('VSwitchId')
        if m.get('VpcAZone') is not None:
            self.vpc_azone = m.get('VpcAZone')
        if m.get('VpcId') is not None:
            self.vpc_id = m.get('VpcId')
        if m.get('VpcRegionId') is not None:
            self.vpc_region_id = m.get('VpcRegionId')
        return self


class CreateDatabaseAccessPointResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class CreateDatabaseAccessPointResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: CreateDatabaseAccessPointResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = CreateDatabaseAccessPointResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class CreateGatewayRequest(TeaModel):
    def __init__(
        self,
        gateway_desc: str = None,
        gateway_name: str = None,
        region_id: str = None,
    ):
        self.gateway_desc = gateway_desc
        self.gateway_name = gateway_name
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_desc is not None:
            result['GatewayDesc'] = self.gateway_desc
        if self.gateway_name is not None:
            result['GatewayName'] = self.gateway_name
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayDesc') is not None:
            self.gateway_desc = m.get('GatewayDesc')
        if m.get('GatewayName') is not None:
            self.gateway_name = m.get('GatewayName')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class CreateGatewayResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class CreateGatewayResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: CreateGatewayResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = CreateGatewayResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class CreateGatewayVerifyCodeRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
    ):
        self.gateway_id = gateway_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        return self


class CreateGatewayVerifyCodeResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class CreateGatewayVerifyCodeResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: CreateGatewayVerifyCodeResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = CreateGatewayVerifyCodeResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DeleteDatabaseRequest(TeaModel):
    def __init__(
        self,
        instance_id: str = None,
    ):
        self.instance_id = instance_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.instance_id is not None:
            result['InstanceId'] = self.instance_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('InstanceId') is not None:
            self.instance_id = m.get('InstanceId')
        return self


class DeleteDatabaseResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DeleteDatabaseResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DeleteDatabaseResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DeleteDatabaseResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DeleteDatabaseAccessPointRequest(TeaModel):
    def __init__(
        self,
        db_instance_id: str = None,
        region_id: str = None,
        v_switch_id: str = None,
        vpc_azone: str = None,
        vpc_id: str = None,
        vpc_region_id: str = None,
    ):
        self.db_instance_id = db_instance_id
        self.region_id = region_id
        self.v_switch_id = v_switch_id
        self.vpc_azone = vpc_azone
        self.vpc_id = vpc_id
        self.vpc_region_id = vpc_region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.db_instance_id is not None:
            result['DbInstanceId'] = self.db_instance_id
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        if self.v_switch_id is not None:
            result['VSwitchId'] = self.v_switch_id
        if self.vpc_azone is not None:
            result['VpcAZone'] = self.vpc_azone
        if self.vpc_id is not None:
            result['VpcId'] = self.vpc_id
        if self.vpc_region_id is not None:
            result['VpcRegionId'] = self.vpc_region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DbInstanceId') is not None:
            self.db_instance_id = m.get('DbInstanceId')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        if m.get('VSwitchId') is not None:
            self.v_switch_id = m.get('VSwitchId')
        if m.get('VpcAZone') is not None:
            self.vpc_azone = m.get('VpcAZone')
        if m.get('VpcId') is not None:
            self.vpc_id = m.get('VpcId')
        if m.get('VpcRegionId') is not None:
            self.vpc_region_id = m.get('VpcRegionId')
        return self


class DeleteDatabaseAccessPointResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DeleteDatabaseAccessPointResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DeleteDatabaseAccessPointResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DeleteDatabaseAccessPointResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DeleteGatewayRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
    ):
        self.gateway_id = gateway_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        return self


class DeleteGatewayResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DeleteGatewayResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DeleteGatewayResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DeleteGatewayResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DeleteGatewayInstanceRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
        gateway_instance_id: str = None,
        region_id: str = None,
    ):
        self.gateway_id = gateway_id
        self.gateway_instance_id = gateway_instance_id
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.gateway_instance_id is not None:
            result['GatewayInstanceId'] = self.gateway_instance_id
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('GatewayInstanceId') is not None:
            self.gateway_instance_id = m.get('GatewayInstanceId')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class DeleteGatewayInstanceResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DeleteGatewayInstanceResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DeleteGatewayInstanceResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DeleteGatewayInstanceResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DescribeRegionsRequest(TeaModel):
    def __init__(
        self,
        region_id: str = None,
    ):
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class DescribeRegionsResponseBodyRegionsRegion(TeaModel):
    def __init__(
        self,
        local_name: str = None,
        region_endpoint: str = None,
        region_id: str = None,
    ):
        self.local_name = local_name
        # Region Endpoint。
        self.region_endpoint = region_endpoint
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.local_name is not None:
            result['LocalName'] = self.local_name
        if self.region_endpoint is not None:
            result['RegionEndpoint'] = self.region_endpoint
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('LocalName') is not None:
            self.local_name = m.get('LocalName')
        if m.get('RegionEndpoint') is not None:
            self.region_endpoint = m.get('RegionEndpoint')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class DescribeRegionsResponseBodyRegions(TeaModel):
    def __init__(
        self,
        region: List[DescribeRegionsResponseBodyRegionsRegion] = None,
    ):
        self.region = region

    def validate(self):
        if self.region:
            for k in self.region:
                if k:
                    k.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        result['Region'] = []
        if self.region is not None:
            for k in self.region:
                result['Region'].append(k.to_map() if k else None)
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        self.region = []
        if m.get('Region') is not None:
            for k in m.get('Region'):
                temp_model = DescribeRegionsResponseBodyRegionsRegion()
                self.region.append(temp_model.from_map(k))
        return self


class DescribeRegionsResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        error_msg: str = None,
        regions: DescribeRegionsResponseBodyRegions = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.error_msg = error_msg
        # Region
        self.regions = regions
        self.request_id = request_id
        self.success = success

    def validate(self):
        if self.regions:
            self.regions.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.regions is not None:
            result['Regions'] = self.regions.to_map()
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('Regions') is not None:
            temp_model = DescribeRegionsResponseBodyRegions()
            self.regions = temp_model.from_map(m['Regions'])
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DescribeRegionsResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DescribeRegionsResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DescribeRegionsResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class DownloadGatewayProgramRequest(TeaModel):
    def __init__(
        self,
        dg_version: str = None,
        user_os: str = None,
    ):
        self.dg_version = dg_version
        self.user_os = user_os

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.dg_version is not None:
            result['DgVersion'] = self.dg_version
        if self.user_os is not None:
            result['UserOS'] = self.user_os
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DgVersion') is not None:
            self.dg_version = m.get('DgVersion')
        if m.get('UserOS') is not None:
            self.user_os = m.get('UserOS')
        return self


class DownloadGatewayProgramResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class DownloadGatewayProgramResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: DownloadGatewayProgramResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = DownloadGatewayProgramResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class FindUserGatewayByIdRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
    ):
        self.gateway_id = gateway_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        return self


class FindUserGatewayByIdResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class FindUserGatewayByIdResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: FindUserGatewayByIdResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = FindUserGatewayByIdResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class GetUserDatabasesRequest(TeaModel):
    def __init__(
        self,
        db_type: str = None,
        gateway_id: str = None,
        host: str = None,
        instance_id: str = None,
        page_number: str = None,
        page_size: str = None,
        port: int = None,
        region_id: str = None,
        search_key: str = None,
    ):
        self.db_type = db_type
        self.gateway_id = gateway_id
        self.host = host
        self.instance_id = instance_id
        self.page_number = page_number
        self.page_size = page_size
        self.port = port
        self.region_id = region_id
        self.search_key = search_key

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.db_type is not None:
            result['DbType'] = self.db_type
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.host is not None:
            result['Host'] = self.host
        if self.instance_id is not None:
            result['InstanceId'] = self.instance_id
        if self.page_number is not None:
            result['PageNumber'] = self.page_number
        if self.page_size is not None:
            result['PageSize'] = self.page_size
        if self.port is not None:
            result['Port'] = self.port
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        if self.search_key is not None:
            result['SearchKey'] = self.search_key
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DbType') is not None:
            self.db_type = m.get('DbType')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('InstanceId') is not None:
            self.instance_id = m.get('InstanceId')
        if m.get('PageNumber') is not None:
            self.page_number = m.get('PageNumber')
        if m.get('PageSize') is not None:
            self.page_size = m.get('PageSize')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        if m.get('SearchKey') is not None:
            self.search_key = m.get('SearchKey')
        return self


class GetUserDatabasesResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        count: int = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: str = None,
    ):
        self.code = code
        self.count = count
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.count is not None:
            result['Count'] = self.count
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Count') is not None:
            self.count = m.get('Count')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class GetUserDatabasesResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: GetUserDatabasesResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = GetUserDatabasesResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class GetUserGatewayInstancesRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
    ):
        self.gateway_id = gateway_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        return self


class GetUserGatewayInstancesResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class GetUserGatewayInstancesResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: GetUserGatewayInstancesResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = GetUserGatewayInstancesResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class GetUserGatewaysRequest(TeaModel):
    def __init__(
        self,
        page_number: int = None,
        page_size: int = None,
        region_id: str = None,
        search_key: str = None,
    ):
        self.page_number = page_number
        self.page_size = page_size
        self.region_id = region_id
        self.search_key = search_key

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.page_number is not None:
            result['PageNumber'] = self.page_number
        if self.page_size is not None:
            result['PageSize'] = self.page_size
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        if self.search_key is not None:
            result['SearchKey'] = self.search_key
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('PageNumber') is not None:
            self.page_number = m.get('PageNumber')
        if m.get('PageSize') is not None:
            self.page_size = m.get('PageSize')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        if m.get('SearchKey') is not None:
            self.search_key = m.get('SearchKey')
        return self


class GetUserGatewaysResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        count: int = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.count = count
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.count is not None:
            result['Count'] = self.count
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Count') is not None:
            self.count = m.get('Count')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class GetUserGatewaysResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: GetUserGatewaysResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = GetUserGatewaysResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class ListDatabaseAccessPointRequest(TeaModel):
    def __init__(
        self,
        db_instance_id: str = None,
        gateway_id: str = None,
        host: str = None,
        page_number: str = None,
        page_size: str = None,
        port: int = None,
        region_id: str = None,
        search_key: str = None,
        vpc_id: str = None,
    ):
        self.db_instance_id = db_instance_id
        self.gateway_id = gateway_id
        self.host = host
        self.page_number = page_number
        self.page_size = page_size
        self.port = port
        self.region_id = region_id
        self.search_key = search_key
        self.vpc_id = vpc_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.db_instance_id is not None:
            result['DbInstanceId'] = self.db_instance_id
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.host is not None:
            result['Host'] = self.host
        if self.page_number is not None:
            result['PageNumber'] = self.page_number
        if self.page_size is not None:
            result['PageSize'] = self.page_size
        if self.port is not None:
            result['Port'] = self.port
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        if self.search_key is not None:
            result['SearchKey'] = self.search_key
        if self.vpc_id is not None:
            result['VpcId'] = self.vpc_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DbInstanceId') is not None:
            self.db_instance_id = m.get('DbInstanceId')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('PageNumber') is not None:
            self.page_number = m.get('PageNumber')
        if m.get('PageSize') is not None:
            self.page_size = m.get('PageSize')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        if m.get('SearchKey') is not None:
            self.search_key = m.get('SearchKey')
        if m.get('VpcId') is not None:
            self.vpc_id = m.get('VpcId')
        return self


class ListDatabaseAccessPointResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        count: int = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: str = None,
    ):
        self.code = code
        self.count = count
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.count is not None:
            result['Count'] = self.count
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Count') is not None:
            self.count = m.get('Count')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class ListDatabaseAccessPointResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: ListDatabaseAccessPointResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = ListDatabaseAccessPointResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class ModifyDatabaseRequest(TeaModel):
    def __init__(
        self,
        db_description: str = None,
        db_name: str = None,
        db_password: str = None,
        db_type: str = None,
        db_user_name: str = None,
        host: str = None,
        instance_id: str = None,
        port: int = None,
    ):
        self.db_description = db_description
        self.db_name = db_name
        self.db_password = db_password
        self.db_type = db_type
        self.db_user_name = db_user_name
        self.host = host
        self.instance_id = instance_id
        self.port = port

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.db_description is not None:
            result['DbDescription'] = self.db_description
        if self.db_name is not None:
            result['DbName'] = self.db_name
        if self.db_password is not None:
            result['DbPassword'] = self.db_password
        if self.db_type is not None:
            result['DbType'] = self.db_type
        if self.db_user_name is not None:
            result['DbUserName'] = self.db_user_name
        if self.host is not None:
            result['Host'] = self.host
        if self.instance_id is not None:
            result['InstanceId'] = self.instance_id
        if self.port is not None:
            result['Port'] = self.port
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('DbDescription') is not None:
            self.db_description = m.get('DbDescription')
        if m.get('DbName') is not None:
            self.db_name = m.get('DbName')
        if m.get('DbPassword') is not None:
            self.db_password = m.get('DbPassword')
        if m.get('DbType') is not None:
            self.db_type = m.get('DbType')
        if m.get('DbUserName') is not None:
            self.db_user_name = m.get('DbUserName')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('InstanceId') is not None:
            self.instance_id = m.get('InstanceId')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        return self


class ModifyDatabaseResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class ModifyDatabaseResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: ModifyDatabaseResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = ModifyDatabaseResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class ModifyGatewayRequest(TeaModel):
    def __init__(
        self,
        gateway_desc: str = None,
        gateway_id: str = None,
        gateway_name: str = None,
    ):
        self.gateway_desc = gateway_desc
        self.gateway_id = gateway_id
        self.gateway_name = gateway_name

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_desc is not None:
            result['GatewayDesc'] = self.gateway_desc
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.gateway_name is not None:
            result['GatewayName'] = self.gateway_name
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayDesc') is not None:
            self.gateway_desc = m.get('GatewayDesc')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('GatewayName') is not None:
            self.gateway_name = m.get('GatewayName')
        return self


class ModifyGatewayResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class ModifyGatewayResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: ModifyGatewayResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = ModifyGatewayResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class RetryDatabaseRequest(TeaModel):
    def __init__(
        self,
        client_token: str = None,
        db_description: str = None,
        db_name: str = None,
        db_password: str = None,
        db_type: str = None,
        db_user_name: str = None,
        gateway_id: str = None,
        host: str = None,
        port: int = None,
        region_id: str = None,
    ):
        self.client_token = client_token
        self.db_description = db_description
        self.db_name = db_name
        self.db_password = db_password
        self.db_type = db_type
        self.db_user_name = db_user_name
        self.gateway_id = gateway_id
        self.host = host
        self.port = port
        self.region_id = region_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.client_token is not None:
            result['ClientToken'] = self.client_token
        if self.db_description is not None:
            result['DbDescription'] = self.db_description
        if self.db_name is not None:
            result['DbName'] = self.db_name
        if self.db_password is not None:
            result['DbPassword'] = self.db_password
        if self.db_type is not None:
            result['DbType'] = self.db_type
        if self.db_user_name is not None:
            result['DbUserName'] = self.db_user_name
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.host is not None:
            result['Host'] = self.host
        if self.port is not None:
            result['Port'] = self.port
        if self.region_id is not None:
            result['RegionId'] = self.region_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('ClientToken') is not None:
            self.client_token = m.get('ClientToken')
        if m.get('DbDescription') is not None:
            self.db_description = m.get('DbDescription')
        if m.get('DbName') is not None:
            self.db_name = m.get('DbName')
        if m.get('DbPassword') is not None:
            self.db_password = m.get('DbPassword')
        if m.get('DbType') is not None:
            self.db_type = m.get('DbType')
        if m.get('DbUserName') is not None:
            self.db_user_name = m.get('DbUserName')
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('Host') is not None:
            self.host = m.get('Host')
        if m.get('Port') is not None:
            self.port = m.get('Port')
        if m.get('RegionId') is not None:
            self.region_id = m.get('RegionId')
        return self


class RetryDatabaseResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class RetryDatabaseResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: RetryDatabaseResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = RetryDatabaseResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


class StopGatewayRequest(TeaModel):
    def __init__(
        self,
        gateway_id: str = None,
        gateway_instance_id: str = None,
    ):
        self.gateway_id = gateway_id
        self.gateway_instance_id = gateway_instance_id

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.gateway_id is not None:
            result['GatewayId'] = self.gateway_id
        if self.gateway_instance_id is not None:
            result['GatewayInstanceId'] = self.gateway_instance_id
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('GatewayId') is not None:
            self.gateway_id = m.get('GatewayId')
        if m.get('GatewayInstanceId') is not None:
            self.gateway_instance_id = m.get('GatewayInstanceId')
        return self


class StopGatewayResponseBody(TeaModel):
    def __init__(
        self,
        code: str = None,
        data: str = None,
        error_msg: str = None,
        request_id: str = None,
        success: bool = None,
    ):
        self.code = code
        self.data = data
        self.error_msg = error_msg
        self.request_id = request_id
        self.success = success

    def validate(self):
        pass

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.code is not None:
            result['Code'] = self.code
        if self.data is not None:
            result['Data'] = self.data
        if self.error_msg is not None:
            result['ErrorMsg'] = self.error_msg
        if self.request_id is not None:
            result['RequestId'] = self.request_id
        if self.success is not None:
            result['Success'] = self.success
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('Code') is not None:
            self.code = m.get('Code')
        if m.get('Data') is not None:
            self.data = m.get('Data')
        if m.get('ErrorMsg') is not None:
            self.error_msg = m.get('ErrorMsg')
        if m.get('RequestId') is not None:
            self.request_id = m.get('RequestId')
        if m.get('Success') is not None:
            self.success = m.get('Success')
        return self


class StopGatewayResponse(TeaModel):
    def __init__(
        self,
        headers: Dict[str, str] = None,
        status_code: int = None,
        body: StopGatewayResponseBody = None,
    ):
        self.headers = headers
        self.status_code = status_code
        self.body = body

    def validate(self):
        self.validate_required(self.headers, 'headers')
        self.validate_required(self.status_code, 'status_code')
        self.validate_required(self.body, 'body')
        if self.body:
            self.body.validate()

    def to_map(self):
        _map = super().to_map()
        if _map is not None:
            return _map

        result = dict()
        if self.headers is not None:
            result['headers'] = self.headers
        if self.status_code is not None:
            result['statusCode'] = self.status_code
        if self.body is not None:
            result['body'] = self.body.to_map()
        return result

    def from_map(self, m: dict = None):
        m = m or dict()
        if m.get('headers') is not None:
            self.headers = m.get('headers')
        if m.get('statusCode') is not None:
            self.status_code = m.get('statusCode')
        if m.get('body') is not None:
            temp_model = StopGatewayResponseBody()
            self.body = temp_model.from_map(m['body'])
        return self


